package main

import (
    "log"
    "fmt"
    "net/http"
    "errors"

    "github.com/wader/goutubedl"
)

func getRedirectURL(input string) (string, error) {
    request, err := http.NewRequest("GET", input, nil)
    if err != nil {
        return "", err
    }

    client := &http.Client{}
    resp, err := client.Do(request)
    if err != nil {
        return "", err
    }

    log.Printf("Found new URL: %s", resp.Request.URL.String())
    return resp.Request.URL.String(), nil
}

func getLargestFormat(input goutubedl.Result, sizeLimit float64) (format goutubedl.Format, err error) {
    for i := len(input.Formats())-1; i >= 0; i-- {
        var size float64
        if input.Formats()[i].FilesizeApprox > 0 {
            size = input.Formats()[i].FilesizeApprox
        } else {
            size = input.Formats()[i].Filesize
        }
        if size <= sizeLimit {
            return input.Formats()[i], nil
        }
    }
    return goutubedl.Format{}, errors.New(fmt.Sprintf("Could not find a video format that is under the size limit of %f", sizeLimit))
}

// Cringe YouTube
func getLargestDashFormat(input goutubedl.Result, sizeLimit float64) (videoFormat goutubedl.Format, audioFormat goutubedl.Format, err error) {
    // Get best audio format
    var bestAudio goutubedl.Format
    for i := len(input.Formats())-1; i >= 0; i-- {
        if input.Formats()[i].ACodec == "opus" {
            bestAudio = input.Formats()[i]
            break
        }
    }

    // Get largest video that will fit in the limit with the audio
    for i := len(input.Formats())-1; i >= 0; i-- {
        if input.Formats()[i].VCodec == "vp9" {
            var size float64
            if input.Formats()[i].FilesizeApprox > 0 {
                size = input.Formats()[i].FilesizeApprox + bestAudio.FilesizeApprox
            } else {
                size = input.Formats()[i].Filesize + bestAudio.Filesize
            }
            if size <= sizeLimit {
                return input.Formats()[i], bestAudio, nil
            }
        }
    }

    return goutubedl.Format{}, goutubedl.Format{}, errors.New(fmt.Sprintf("Could not find a video format that is under the size limit of %f", sizeLimit))
}

